function melody_yin(wavfile,transfile,step)

% MELODY_YIN Melody estimation via loudness-weighted YIN.
%
% melody_yin(wavfile,transfile)
% melody_yin(wavfile,transfile,step)
%
% Inputs:
% wavfile: input WAV filename
% transfile: output f0 filename
% step: stepsize in s (default: 0.0223)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright 2004-2005 Emmanuel Vincent (Queen Mary University of London)
% This software is distributed under the terms of the GNU Public License
% version 3 (http://www.gnu.org/licenses/gpl.txt)
% If you find it useful, please cite the following reference:
% Emmanuel Vincent, "Predominant-F0 estimation using Bayesian harmonic
% waveform models", in Proc. MIREX, 2005.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%lecture du fichier son
[x,fs]=wavread(wavfile);
x=resample(x,22050,fs);

%parametres fixes manuellement
N=single(1024);
if nargin<3,
    S=N/2;
else
    S=22050*step;
end
x=single([zeros(N/2,1);x;zeros(N/2,1)]);
T=single(length(x)); x=norm_sound(x(:));
win=hanning(N);
nfram=floor((T-N)/S)+1;

firstnote=single(45); lastnote=single(84);
ups=single(5);   %facteur d'upsampling de l'echelle MIDI
noterange=lastnote+1:-1/ups:firstnote-1; nbnotes=length(noterange);
dvect=single(22050/440./2.^((noterange-69)/12));

%calcul de la reponse de l'oreille externe
freqs=[0,20,25,31.5,40,50,63,80,100,125,160,200,250,315,400,500,630,800,1000,1250,1600,2000,2500,3150,4000,5000,6300,8000,9000,10000,12500,15000,20000];
datamaf=[75.8,70.1,60.8,52.1,44.2,37.5,31.3,25.6,20.9,16.5,12.6,9.6,7.0,4.7,3.0,1.8,0.8,0.2,0.0,-0.5,-1.6,-3.2,-5.4,-7.8,-8.1,-5.3,2.4,11.1,12.8,12.2,7.4,17.8,17.8];
earfilt=-interp1(freqs/22050,datamaf,0:1/N:.5,'spline');
earfilt=single(10.^(earfilt.'/10));

%earfilt=ones(N/2+1,1);

%calcul des masques frequentiels
mask=zeros(nbnotes,N/2+1);
for d=1:nbnotes,
        mask(d,:)=1-exp(2*1i*pi*(0:N/2)/N*dvect(d));
end
mask=abs(mask).^2;

%calcul de f0 sur chaque trame
f0=zeros(1,nfram,'single');
for t=1:nfram,
    y=x(round((t-1)*S)+1:round((t-1)*S)+N).*win;
    y=fft(y); y=abs(y(1:N/2+1)).^2;
    df=mask*(y.*earfilt);
    [val,ind]=min(df);
    if (ind > 1)&&(ind < nbnotes),
        coeffs=polyfit(dvect(ind-1:ind+1).',df(ind-1:ind+1),2);
        f0(t)=-2*coeffs(1)/coeffs(2)*22050;
    end
end

%ecriture du fichier transcription
time=(0:S:(nfram-1)*S)/22050;
txt='';
for t=1:nfram,
    txt=[txt num2str(time(t)) char(9) num2str(f0(t)) char(10)];
end
fid=fopen(transfile,'w');
fprintf(fid,'%c',txt);
fclose(fid);

return;



function [s,coeff]=norm_sound(s)

%normalise un son a une puissance maximale de -12 dB calculee sur des fenetres rectangulaires de taille 256

[T nchan]=size(s);
if (nchan > T), s=s.'; end;
en=reshape([sum(s.^2,2);zeros(ceil(T/256)*256-T,1)],256,ceil(T/256));
en=max(sum(en))/256;
coeff=.25/sqrt(en);
s=s*coeff;

return;
